use crate::egui::{Align2, Context};
use crate::{ctx_img, img, Margin, Slide};
use eframe::egui;
use eframe::egui::{Align, Color32, Frame, Layout, ScrollArea, Style, Ui, Vec2, Window};
use rand::Rng;

#[derive(Default)]
pub struct Title {
    examples: Vec<egui::TextureHandle>,
}

impl Slide for Title {
    fn show(&mut self, ui: &mut Ui, ctx: &Context) {
        if self.examples.is_empty() {
            self.examples = vec![
                ctx_img!(ui.ctx(), "raymarching0.png"),
                ctx_img!(ui.ctx(), "raymarching1.png"),
                ctx_img!(ui.ctx(), "atom0.png"),
                ctx_img!(ui.ctx(), "atom1.png"),
                ctx_img!(ui.ctx(), "atom2.png"),
            ];
        }

        ui.scope(|ui| {
            ui.style_mut().spacing.window_margin = Margin::same(4.0);

            for example in &self.examples {
                Window::new(example.name())
                    .title_bar(false)
                    .resizable(false)
                    .frame(Frame::window(&ui.style()).margin(Margin::same(5.0)))
                    .show(ctx, |ui| {
                        ui.image(example, Vec2::splat(128.0));
                    });
            }
        });

        Window::new("title")
            .title_bar(false)
            .resizable(false)
            .anchor(Align2::CENTER_CENTER, Vec2::ZERO)
            .show(ctx, |ui| {
                ui.vertical_centered(|ui| {
                    ui.heading("Generative Art");
                    ui.label("By: TODO");
                });
            });
    }
}
