use crate::egui::{Context, Ui, Window};
use crate::{Key, Slide};
use eframe::egui::text_edit::{CCursorRange, CursorRange, TextEditState};
use eframe::egui::{Align, Align2, Frame, Layout, TextEdit, Vec2};
use eframe::epaint::text::cursor::{CCursor, Cursor};
use std::ops::RangeInclusive;

#[derive(Default)]
pub struct Complexity {
    state: ComplexityState,
    last_time: f64,
}

#[derive(Default, Eq, PartialEq, Debug)]
enum ComplexityState {
    #[default]
    Before,
    Selecting(usize),
    Typing(usize),
    After,
}

impl Slide for Complexity {
    fn transition(&mut self, ctx: &Context) -> bool {
        if self.state == ComplexityState::Before {
            self.state = ComplexityState::Selecting(0);
            false
        } else {
            true
        }
    }

    fn show(&mut self, ui: &mut Ui, ctx: &Context) {
        Window::new("complexity")
            .title_bar(false)
            .resizable(false)
            .frame(Frame::none())
            .anchor(Align2::CENTER_CENTER, Vec2::ZERO)
            .fixed_size(Vec2::new(440.0, 100.0))
            .show(ctx, |ui| {
                ui.ctx().request_repaint();
                let time = ui.input().time;
                let time_delta = (time - self.last_time).min(1.0);
                let made_progress = time_delta > 0.1;
                if made_progress {
                    self.last_time = time;
                }

                ui.vertical_centered_justified(|ui| {
                    let mut state = TextEditState::default();
                    let text;
                    match &mut self.state {
                        ComplexityState::Before => {
                            text = String::from("Complex Rules ➡ Complex Results?");
                        }
                        ComplexityState::Selecting(progress) => {
                            text = String::from("Complex Rules ➡ Complex Results?");
                            state.set_ccursor_range(Some(CCursorRange::two(
                                CCursor::new(0),
                                CCursor::new(*progress),
                            )));
                            if made_progress {
                                if *progress == "Complex".len() {
                                    self.state = ComplexityState::Typing(0);
                                } else {
                                    *progress += 1;
                                }
                            }
                        }
                        ComplexityState::Typing(progress) => {
                            text = format!(" {} Rules ➡ Complex Results?", &"Simple"[0..*progress]);
                            if made_progress {
                                if *progress == "Simple".len() {
                                    self.state = ComplexityState::After;
                                } else {
                                    *progress += 1;
                                }
                            }
                        }
                        ComplexityState::After => {
                            text = String::from(" Simple Rules ➡ Complex Results!");
                        }
                    }

                    let text_edit = ui.text_edit_singleline(&mut text.as_str());
                    if state.ccursor_range().is_some() {
                        text_edit.request_focus();
                    } else {
                        text_edit.surrender_focus();
                    }
                    TextEdit::store_state(ctx, text_edit.id, state);
                });
            });
    }
}
