use crate::component::grid::Grid;
use crate::governor::Governor;
use crate::Slide;
use eframe::egui::style::Margin;
use eframe::egui::{Align2, Color32, Context, Frame, Pos2, Rect, Shape, Stroke, Ui, Vec2, Window};
use eframe::emath;
use rand::Rng;

pub struct Conway {
    life: Grid,
    governor: Governor,
}

impl Default for Conway {
    fn default() -> Self {
        let mut life = Grid::default();

        for y in 0..life.size_cells {
            for x in 0..life.size_cells {
                if rand::thread_rng().gen_bool(0.5) {
                    life.set_fill(x, y, Color32::BLACK);
                }
            }
        }

        /*
        life.clear_fill();
        life.set_fill(5, 5, Color32::BLACK);
        life.set_fill(6, 5, Color32::BLACK);
        life.set_fill(7, 5, Color32::BLACK);
         */

        Self {
            life,
            governor: Governor::default(),
        }
    }
}

impl Slide for Conway {
    fn show(&mut self, ui: &mut Ui, ctx: &Context) {
        Frame::none().margin(Margin::same(20.0)).show(ui, |ui| {
            ui.vertical_centered(|ui| {
                ui.heading("Cellular Automata");
            });
        });

        ctx.request_repaint();
        if self.governor.ready(ctx.input().time, 0.2) {
            self.life = conway(&self.life);
        }
        self.life.show(ctx);
    }
}

fn conway(grid: &Grid) -> Grid {
    let mut new = grid.clone();
    new.clear_fill();

    for cy in 0..grid.size_cells {
        for cx in 0..grid.size_cells {
            let mut neighbors = 0;
            for dy in -1..=1 {
                for dx in -1..=1 {
                    let (x, y) = match (cx.checked_add_signed(dx), cy.checked_add_signed(dy)) {
                        (Some(x), Some(y)) if x < grid.size_cells && y < grid.size_cells => (x, y),
                        _ => continue,
                    };
                    if x == cx && y == cy {
                        // Don't consider self.
                        continue;
                    }
                    if grid.fill(x, y).a() > 0 {
                        neighbors += 1;
                    }
                }
            }

            let mut alive = grid.fill(cx, cy).a() > 0;

            alive = match (alive, neighbors) {
                (false, 3) => true,
                (false, _) => false,
                (true, 2) | (true, 3) => true,
                (true, _) => false,
            };

            new.set_fill(
                cx,
                cy,
                if alive {
                    Color32::BLACK
                } else {
                    Color32::TRANSPARENT
                },
            );
        }
    }

    new
}
